﻿---------------------------------------------------------
--	Library
---------------------------------------------------------
local BZ = LibStub("LibBabble-Zone-3.0"):GetLookupTable()

if not _G.BZ then
	_G.BZ = bzone:GetLookupTable()
end


---------------------------------------------------------
--	Localization
---------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("GridStatusRaidDebuff")

L:RegisterTranslations("enUS", function()
	return {		
		["Raid Debuff"] = true,
		["Option for %s"] = true,
		["Enable %s"] = true,
	}
end)

L:RegisterTranslations("koKR", function()
	return {
		["Raid Debuff"] = "레이드 디버프",
		["Option for %s"] = "%s의 설정",
		["Enable %s"] = "%s 활성화",
	}
end)

L:RegisterTranslations("zhCN", function()
	return {
		["Raid Debuff"] = "团队减益",
		["Option for %s"] = "%s 设置",
		["Enable %s"] = "开启 %s",
	}	
end)

L:RegisterTranslations("zhTW", function()
	return {
		["Raid Debuff"] = "團隊減益",
		["Option for %s"] = "%s 選項",
		["Enable %s"] = "啟用%s",
	}
end)

L:RegisterTranslations("ruRU", function()
	return {
		["Raid Debuff"] = "Дебаффы рейда",
		["Option for %s"] = "Опции %s",
		["Enable %s"] = "Включить %s",
	}
end)


---------------------------------------------------------
--	Core
---------------------------------------------------------
GSRD = GridStatus:NewModule("GridStatusRaidDebuff")

local GSRD = GSRD
GSRD.menuName = L["Raid Debuff"]

local GetSpellInfo = GetSpellInfo
local fmt = string.format
local bit_band = bit.band

local MemberStatuses = {}
local debuff_list = {}

local AuraCheckListZones = {}
local AuraCheckListAuras = {}

local zone
local db

GSRD.defaultDB = {
	debug = false,	
	
	alert_RaidDebuff = {
		text = L["Raid Debuff"],
		desc = L["Raid Debuff"],
		enable = true,
		color = { r = 0.0, g = 0.0, b = 0.0, a = 1 },
		priority = 98,
		range = false,
	},
	
	debuff_option = {}
}


function GSRD:OnInitialize()
	self.super.OnInitialize(self)
	self:RegisterStatuses()		
end

function GSRD:OnEnable()
	self.debugging = self.db.profile.debug
	
	db = self.db.profile.debuff_option

	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneCheck")
end

function GSRD:Grid_UnitJoined(guid, unitid)
	self:ScanUnitAuras(unitid)
end

function GSRD:ZoneCheck()
	zone = GetRealZoneText()

	self:NukeStatuses()

	if debuff_list[zone] then
		if not self:IsEventRegistered("COMBAT_LOG_EVENT_UNFILTERED") then
			self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "EventHandler")
			self:RegisterEvent("PLAYER_REGEN_ENABLED")

			-- If this zone has debuffs that require aura checking, then register for
			-- the necessary events.
			if (AuraCheckListZones[zone]) then
				if not self:IsEventRegistered("UNIT_AURA") then
					self:RegisterEvent("UNIT_AURA", "ScanUnitAuras")
					self:RegisterEvent("Grid_UnitJoined")
				end
			else
				if self:IsEventRegistered("UNIT_AURA") then
					self:UnregisterEvent("UNIT_AURA")
					self:UnregisterEvent("Grid_UnitJoined")
				end
			end
		end
	else
		if self:IsEventRegistered("COMBAT_LOG_EVENT_UNFILTERED") then
			self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
			self:UnregisterEvent("PLAYER_REGEN_ENABLED")

			if self:IsEventRegistered("UNIT_AURA") then
				self:UnregisterEvent("UNIT_AURA")
				self:UnregisterEvent("Grid_UnitJoined")
			end
		end
	end
end

function GSRD:NukeStatuses()
	for guid in pairs(MemberStatuses) do
		MemberStatuses[guid] = nil
		self.core:SendStatusLost(guid, "alert_RaidDebuff")
	end
end

function GSRD:NukeUnit(guid)
	if (MemberStatuses and MemberStatuses[guid]) then
		MemberStatuses[guid] = nil
		self.core:SendStatusLost(guid, "alert_RaidDebuff")
	end
end

function GSRD:PLAYER_REGEN_ENABLED()
	for guid, unitid in GridRoster:IterateRoster() do
		if not UnitDebuff(unitid, 1) then
			local uName = UnitName(unitid)
			self:NukeUnit(guid)
		end
	end
end

function GSRD:PLAYER_ENTERING_WORLD()
	if not zone then
		self:ZoneCheck()
	else
		self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	end
end

function GSRD:RegisterStatuses()
	self:RegisterStatus("alert_RaidDebuff", L["Raid Debuff"], nil, true)
end

function GSRD:UnregisterStatuses()
	self:UnregisterStatus("alert_RaidDebuff")
end

-- Create a custom tooltip for debuff description
local tip = CreateFrame("GameTooltip", "GridStatusRaidDebuffTooltip", nil, "GameTooltipTemplate")
tip:SetOwner(UIParent, "ANCHOR_NONE")
for i = 1, 10 do
	tip[i] = _G["GridStatusRaidDebuffTooltipTextLeft"..i]
	if not tip[i] then
		tip[i] = tip:CreateFontString()
		tip:AddFontStrings(tip[i], tip:CreateFontString())
	end
end


function GSRD:Reset()
	self.super.Reset(self)
	self:UnregisterStatuses()
	self:RegisterStatuses()
end


function GSRD:GainedDebuff(guid, stNum, count)
	if debuff_list[zone] then
		local dInfo = debuff_list[zone][stNum]
		if dInfo then
			if dInfo.delegater then
				dInfo = debuff_list[zone][dInfo.delegater]
			end

			if not db[zone][stNum] then
				if not MemberStatuses[guid] then MemberStatuses[guid] = {} end

				MemberStatuses[guid][stNum] = {
					value = count and fmt("%d",count),
					color = dInfo.color or self.db.profile["alert_RaidDebuff"].color,
					icon = dInfo.icon,
					start = GetTime(),
					duration = dInfo.duration,
					stackable = dInfo.stackable,					
				}
				
				self:UpdateUnit(guid)
			end
		end
	end
end


-- COMBAT_LOG_EVENT_UNFILTERED handler
function GSRD:EventHandler(ts, event, srcguid, srcname, srcflg, dstguid, dstname, dstflg, ...)
	if bit_band(dstflg, 0x00006868) == 0 then
		if
			event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_APPLIED_DOSE" or event == "SPELL_AURA_REMOVED_DOSE" or event == "SPELL_AURA_REFRESH"
		then
			local spellId, name, _, _, count = select(1, ...)						
			self:GainedDebuff(dstguid, spellId, count or 1)			
		elseif
			event == "SPELL_AURA_REMOVED"
		then
			local spellId, name = select(1, ...)			
			self:LostDebuff(dstguid, spellId)			
		elseif
			event == "UNIT_DIED"
		then
			self:NukeUnit(dstguid)
		end
	end
end

function GSRD:LostDebuff(guid, stNum)
	if (MemberStatuses and MemberStatuses[guid] and MemberStatuses[guid][stNum]) then
		MemberStatuses[guid][stNum] = nil
		self:UpdateUnit(guid)
	end
end


function GSRD:UpdateUnit(guid)
	if (MemberStatuses[guid] and next(MemberStatuses[guid])) then
		local settings = self.db.profile["alert_RaidDebuff"]
		local icon, start, duration
		local value, stackicon

		local normalcolor
		local stackcolor
		for i, k in pairs(MemberStatuses[guid]) do
			if k.stackable then
				stackicon = k.icon
				value = k.value
				stackcolor = k.color
			end
			if not duration then
				icon = k.icon
				start = k.start
				duration = k.duration
				normalcolor = k.color
			end
		end

		
		
		local	color = stackcolor or normalcolor

		self.core:SendStatusGained(
			guid, "alert_RaidDebuff", settings.priority, (settings.range and 40),
			color, nil, nil, nil, icon or stackicon, start, duration, value)
	else
		self.core:SendStatusLost(guid, "alert_RaidDebuff")
	end
end


local debuff_names_seen = {}

function GSRD:ScanUnitAuras(unitid)
	local name, rank, icon, count, debuffType, duration, expirationTime, caster, isStealable

	local guid = UnitGUID(unitid)
	if not GridRoster:IsGUIDInRaid(guid) then
		return
	end

	self:Debug("UNIT_AURA", unitid, guid)

	-- scan for debuffs
	wipe(debuff_names_seen)
	local settings = self.db.profile["alert_RaidDebuff"]
	if (settings.enable) then
		local index = 1
		while true do
			name, rank, icon, count, debuffType, duration, expirationTime, caster, isStealable = UnitAura(unitid, index, "HARMFUL")

			if not name then
				break
			end

			if (AuraCheckListAuras[name]) then
				debuff_names_seen[name] = true
				self:GainedDebuff(guid, AuraCheckListAuras[name], count or 1)
			end

			index = index + 1
		end
	end

	-- handle lost debuffs
	for name in pairs(AuraCheckListAuras) do
		if not debuff_names_seen[name] then
			self:LostDebuff(guid, AuraCheckListAuras[name])
		else
			debuff_names_seen[name] = nil
		end
	end
end

function GSRD:RegisterDebuff(zone, debuffID, order, duration, stackable, disable, auraCheck, color)
	local db = self.db.profile.debuff_option
	
	if not db[zone] then db[zone] = {}	end	
	if disable then	db[zone][debuffID] = disable	end
	
	local args
	if not debuff_list[zone] then 
		debuff_list[zone] = {}
		
		args = GridStatus.options.args["alert_RaidDebuff"].args
		args[zone] = {
			type = "group",
			name = zone,
			desc = L["Option for %s"]:format(zone),			
			args = {},
		}
	end
	
	if not debuff_list[zone][debuffID] then
		debuff_list[zone][debuffID] = {}
		
		local name, _, icon = GetSpellInfo(debuffID)	
		
		local description = L["Enable %s"]:format(name or debuffID)
		tip:SetHyperlink("spell:"..debuffID)
		if tip:NumLines() > 1 then
			description = tip[tip:NumLines()]:GetText()
		end
		
		debuff_list[zone][debuffID].name = name	
		debuff_list[zone][debuffID].icon = icon
		
		args = GridStatus.options.args["alert_RaidDebuff"].args[zone].args
		args[debuffID] = {
			type = "toggle",
			name = name,
			desc = description,
			order = order,
			get = function() return not db[zone][debuffID] end,
			set = function(v) db[zone][debuffID] = not v end,
		}
	end

	local data = debuff_list[zone][debuffID]
		
	data.stackable = stackable
	data.duration = duration	
	
	if color then
		data.color = color
	end
	
	if auraCheck then	
		AuraCheckListZones[zone] = true
		AuraCheckListAuras[data.name] = debuffID
	end
	
end

function GSRD:RegisterDebuffDelegater(zone, debuffID, ...)
	local n = select("#",...)
	if n > 0 then
		for i = 1,n do
			debuff_list[zone][select(i,...)] = { delegater = debuffID }
		end
	end
end